@@ -28,7 +28,6 @@
 #include "afterconf.h"
 
 
-void          ProcessStatement (ConfigDef * config);
 /*****************************************************************************
  *
  * This routine is responsible for reading and parsing the base.<bpp> config
@@ -38,6 +37,57 @@ void          ProcessStatement (ConfigDef * config);
 
 #define WHARF_FOLDER_END 	"~Folder"
 
+TermDef       WharfHintsTerms[] = {
+    {TF_NO_MYNAME_PREPENDING, "Name", 4,        TT_FLAG, WHINT_Name_ID     		, NULL},
+    {TF_NO_MYNAME_PREPENDING, "Comment", 7,     TT_FLAG, WHINT_Comment_ID     	, NULL},
+    {TF_NO_MYNAME_PREPENDING, "Exec", 4,        TT_FLAG, WHINT_Exec_ID		    , NULL},
+    {TF_NO_MYNAME_PREPENDING, "GenericName", 11,TT_FLAG, WHINT_GenericName_ID   , NULL},
+    {0, NULL, 0, 0, 0}
+};
+
+SyntaxDef     WharfHintsSyntax = {
+	',',
+	'\n',
+    WharfHintsTerms,
+	0,										   /* use default hash size */
+    ' ',
+	" ",
+	"\t",
+    "Wharf Show Hints flags",
+	"Wharf Hints",
+	"",
+	NULL,
+	0
+};
+struct SyntaxDef     *WharfHintsSyntaxPtr = &WharfHintsSyntax;
+
+flag_options_xref WharfHintsFlagsXref[] = {
+    {WHARF_SHOW_HINT_Name, WHINT_Name_ID, 0},
+    {WHARF_SHOW_HINT_Comment, WHINT_Comment_ID, 0},
+    {WHARF_SHOW_HINT_Exec, WHINT_Exec_ID, 0},
+    {WHARF_SHOW_HINT_GenericName, WHINT_GenericName_ID, 0},
+    {0, 0, 0}
+};
+
+
+
+
+ASFlagType
+ParseWharfHintsOptions( FreeStorageElem * options )
+{
+    ASFlagType hints = 0 ;
+    while( options )
+	{
+        LOCAL_DEBUG_OUT( "options(%p)->keyword(\"%s\")", options, options->term->keyword );
+        if (options->term != NULL)
+            ReadFlagItem (NULL, &hints, options, WharfHintsFlagsXref);
+        options = options->next;
+    }
+    return hints;
+}
+
+
+
 TermDef       WhevTerms[] = {
 	{TF_NO_MYNAME_PREPENDING | TF_SYNTAX_TERMINATOR, "push", 4, TT_FILENAME, WHEV_PUSH_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING | TF_SYNTAX_TERMINATOR, "close_folder", 12, TT_FILENAME, WHEV_CLOSE_FOLDER_ID, NULL},
@@ -73,13 +123,18 @@ SyntaxDef     WhevSyntax = {
 
 #define WHARF_LOOK_TERMS \
 	{0, "ShowLabel", 9,         TT_FLAG, WHARF_ShowLabel_ID, NULL}, \
-    {0, "LabelLocation", 13,    TT_UINTEGER, WHARF_LabelLocation_ID, NULL}, \
-    {0, "FlipLabel", 9,         TT_FLAG, WHARF_FlipLabel_ID, NULL}, \
-    {0, "FitContents", 11,      TT_FLAG, WHARF_FitContents_ID, NULL}, \
-    {0, "ShapeToContents", 15,  TT_FLAG, WHARF_ShapeToContents_ID, NULL}, \
-    {0, "AlignContents", 13,    TT_FLAG, WHARF_AlignContents_ID, &AlignSyntax}, \
-    {0, "Bevel", 5, 			TT_FLAG, WHARF_Bevel_ID, &BevelSyntax}, \
-    {0, "CompositionMethod", 17,TT_INTEGER, WHARF_CompositionMethod_ID, NULL}
+    ASCF_DEFINE_KEYWORD(WHARF, 0, LabelLocation		, TT_UINTEGER	, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, FlipLabel			, TT_FLAG		, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, FitContents		, TT_FLAG		, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, ShapeToContents	, TT_FLAG		, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, StretchBackground	, TT_FLAG		, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, AlignContents		, TT_FLAG		, &AlignSyntax), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, Bevel				, TT_FLAG		, &BevelSyntax), \
+	ASCF_DEFINE_KEYWORD(WHARF, 0, CompositionMethod	, TT_INTEGER	, NULL), \
+	ASCF_DEFINE_KEYWORD(WHARF, 0, FolderOffset		, TT_INTEGER	, NULL), \
+	ASCF_DEFINE_KEYWORD(WHARF, 0, OrthogonalFolderOffset, TT_INTEGER	, NULL), \
+    ASCF_DEFINE_KEYWORD(WHARF, 0, ShowHints			, TT_FLAG		, &WharfHintsSyntax)
+
 
 #define WHARF_PRIVATE_TERMS \
     {0, "Sound", 5,             TT_FILENAME, WHARF_Sound_ID, &WhevSyntax}, \
@@ -89,8 +144,8 @@ SyntaxDef     WhevSyntax = {
     {0, "ForceSize", 9,         TT_GEOMETRY, WHARF_ForceSize_ID, NULL}
 
 #define WHARF_FOLDER_TERMS \
-	{TF_SPECIAL_PROCESSING, "", 0, TT_SPECIAL, WHARF_Wharf_ID, &FuncSyntax}, \
-	{TF_NO_MYNAME_PREPENDING|TF_SYNTAX_TERMINATOR, WHARF_FOLDER_END, 7, TT_FLAG, WHARF_FolderEnd_ID, NULL}
+	{TF_SPECIAL_PROCESSING|TF_NAMED|TF_USE_TOKENS(1), "", 0, TT_ICON_LIST, WHARF_Wharf_ID, &DummyFuncSyntax}, \
+	{TF_SYNTAX_TERMINATOR, WHARF_FOLDER_END, 7, TT_FLAG, WHARF_FolderEnd_ID, NULL}
 
 
 TermDef       WharfTerms[] = {
@@ -99,7 +154,6 @@ TermDef       WharfTerms[] = {
     {TF_OBSOLETE, "NoWithdraw", 10,       TT_FLAG, WHARF_NoWithdraw_ID, NULL},
 /* TextureType, MaxColors, BgColor, TextureColor, and Pixmap are obsolete */
 	{TF_OBSOLETE, "TextureType", 11, TT_UINTEGER, WHARF_TextureType_ID, NULL},
-    {TF_OBSOLETE, "MaxColors", 9,    TT_UINTEGER, WHARF_MaxColors_ID, NULL},
     {TF_OBSOLETE, "BgColor", 7,      TT_COLOR, WHARF_BgColor_ID, NULL},
     {TF_OBSOLETE, "TextureColor", 12,TT_COLOR, WHARF_TextureColor_ID, NULL},
     {TF_OBSOLETE, "Pixmap", 6,       TT_FILENAME, WHARF_Pixmap_ID, NULL},
@@ -164,18 +218,18 @@ flag_options_xref WharfFlags[] = {
 	{WHARF_ANIMATE_MAIN, WHARF_AnimateMain_ID, 0},
 	{WHARF_ANIMATE, WHARF_Animate_ID, 0},
     {WHARF_SHOW_LABEL, WHARF_ShowLabel_ID, 0},
-    {WHARF_FLIP_LABEL, WHARF_FlipLabel_ID, 0},
-    {WHARF_FIT_CONTENTS, WHARF_FitContents_ID, 0},
-    {WHARF_SHAPE_TO_CONTENTS, WHARF_ShapeToContents_ID, 0 },
+    {WHARF_FlipLabel, WHARF_FlipLabel_ID, 0},
+    {WHARF_FitContents, WHARF_FitContents_ID, 0},
+    {WHARF_ShapeToContents, WHARF_ShapeToContents_ID, 0 },
+    {WHARF_StretchBackground, WHARF_StretchBackground_ID, 0 },
     {0, 0, 0}
 };
 
 WharfButton  *
 CreateWharfButton ()
 {
-	WharfButton  *btn = (WharfButton *) safemalloc (sizeof (WharfButton));
+	WharfButton  *btn = (WharfButton *) safecalloc (1, sizeof (WharfButton));
 
-	memset (btn, 0x00, sizeof (WharfButton));
 	return btn;
 }
 
@@ -192,6 +246,8 @@ DestroyWharfButton (WharfButton **pbtn)
     /* delete members */
 	if (btn->title != NULL)
 		free (btn->title);
+	if (btn->comment != NULL)
+		free (btn->comment);
 
 	if( btn->contents )
 	{
@@ -227,18 +283,21 @@ DestroyWharfButton (WharfButton **pbtn)
 WharfConfig  *
 CreateWharfConfig ()
 {
-	WharfConfig  *config = (WharfConfig *) safemalloc (sizeof (WharfConfig));
+	WharfConfig  *config = (WharfConfig *) safecalloc (1, sizeof (WharfConfig));
 
-	memset (config, 0x00, sizeof (WharfConfig));
 	/* let's initialize Base config with some nice values: */
 	config->geometry.flags = WidthValue | HeightValue;
 	config->geometry.width = config->geometry.height = 64;
-        config->withdraw_style = WITHDRAW_ON_EDGE_BUTTON_AND_SHOW ;
-        config->align_contents = ALIGN_CENTER ;
+    config->withdraw_style = WITHDRAW_ON_EDGE_BUTTON_AND_SHOW ;
+    
+	config->AlignContents = WHARF_DEFAULT_AlignContents ;
+	config->ShowHints = WHARF_DEFAULT_ShowHints ;
 
 	config->more_stuff = NULL;
 
-    config->composition_method = TEXTURE_TRANSPIXMAP_ALPHA ;
+    config->CompositionMethod = WHARF_DEFAULT_CompositionMethod ;
+	config->FolderOffset = WHARF_DEFAULT_FolderOffset ;
+	config->OrthogonalFolderOffset = WHARF_DEFAULT_OrthogonalFolderOffset ;
 
 	return config;
 }
@@ -341,30 +400,33 @@ PrintWharfConfig(WharfConfig *config )
         for( i = 0 ; i < WHEV_MAX_EVENTS ; ++i )
             show_progress( "WHARF.sounds[%d]=\"%s\";", i, config->sounds[i] );
     }
-
-    if( get_flags( config->set_flags, WHARF_LABEL_LOCATION ) )
-        show_progress( "WHARF.label_location = 0x%lX", config->label_location );
-    if( get_flags( config->set_flags, WHARF_ALIGN_CONTENTS ) )
-        show_progress( "WHARF.align_context = 0x%lX;", config->align_contents );
-    if( get_flags( config->set_flags, WHARF_BEVEL ) )
-        show_progress( "WHARF.bevel = 0x%lX;", config->bevel);
-    if( get_flags( config->set_flags, WHARF_COMPOSITION_METHOD ) )
-        show_progress( "WHARF.composition_method = %d;", config->composition_method );
-
-    print_wharf_folder( config->root_folder, 1 );
+	
+	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,LabelLocation);
+	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,FlipLabel);
+	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,FitContents);
+	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,ShapeToContents );
+	ASCF_PRINT_FLAG_KEYWORD(stderr,WHARF,config,StretchBackground );
+	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,AlignContents );
+	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,Bevel );
+	ASCF_PRINT_FLAGS_KEYWORD(stderr,WHARF,config,ShowHints );
+	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,CompositionMethod);
+	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,FolderOffset);
+	ASCF_PRINT_INT_KEYWORD(stderr,WHARF,config,OrthogonalFolderOffset);
+    
+	print_wharf_folder( config->root_folder, 1 );
 
 }
 
 void print_trimmed_str( char *prompt, char * str );
 
 unsigned long
-WharfSpecialFunc (ConfigDef * config, FreeStorageElem ** storage)
+WharfSpecialFunc (ConfigDef * config)
 {
 	TermDef      *pterm;
 	register char *cur;
 
-    LOCAL_DEBUG_CALLER_OUT("%p,%p", config, storage);
-	if (config == NULL || storage == NULL)
+    LOCAL_DEBUG_CALLER_OUT("%p", config);
+	if (config == NULL )
 		return SPECIAL_BREAK;
 
 	/* checking if we have ~Folders in here */
@@ -411,7 +473,7 @@ WharfSpecialFunc (ConfigDef * config, FreeStorageElem ** storage)
 		config->cursor = cur;
         /* we are at the beginning of the function definition right now - lets process it :*/
         /* read in entire function definition */
-		GetNextStatement (config, 1);
+		GetNextStatement (config);
         /* lets find us the term for this definition */
         print_trimmed_str( "config->current_data", config->current_data );
         LOCAL_DEBUG_OUT( "curr_data_len = %d", config->current_data_len);
@@ -613,25 +675,22 @@ ParseWharfOptions (const char *filename, char *myname)
 	FreeStorageElem *Storage = NULL, *pCurr;
 	ConfigItem    item;
 	MyStyleDefinition **styles_tail = &(config->style_defs);
+	TermDef      *folder_term = func2fterm (F_Folder, False);
 
 	cd.filename = filename ;
-	ConfigReader = InitConfigReader (myname, &WharfSyntax, CDT_Filename, cd, WharfSpecialFunc);
+	ConfigReader = InitConfigReader (myname, &WharfSyntax, CDT_Filename, cd, NULL /*WharfSpecialFunc*/);
 	if (!ConfigReader)
 		return config;
 
-    FuncTerms[F_Folder].sub_syntax = &WharfSyntax ;
+    folder_term->sub_syntax = &WharfSyntax ;
 
     item.memory = NULL;
 	PrintConfigReader (ConfigReader);
-SHOW_CHECKPOINT;
 	ParseConfig (ConfigReader, &Storage);
-SHOW_CHECKPOINT;
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &(config->more_stuff), CF_DISABLED_OPTION);
-SHOW_CHECKPOINT;
-	config->balloon_conf = Process_balloonOptions (Storage, NULL);
+	config->balloon_conf = Process_balloonOptions (Storage, NULL, BALLOON_ID_START);
 
 	for (pCurr = Storage; pCurr; pCurr = pCurr->next)
 	{
@@ -680,10 +739,6 @@ SHOW_CHECKPOINT;
 			 set_flags (config->set_flags, WHARF_TEXTURE_TYPE);
 			 config->texture_type = item.data.integer;
 			 break;
-		 case WHARF_MaxColors_ID:
-			 set_flags (config->set_flags, WHARF_MAX_COLORS);
-			 config->max_colors = item.data.integer;
-			 break;
 		 case WHARF_BgColor_ID:
 			 set_string_value (&(config->bg_color), item.data.string, &(config->set_flags), WHARF_BG_COLOR);
 			 break;
@@ -723,23 +778,13 @@ SHOW_CHECKPOINT;
 			 }
 			 item.ok_to_free = 1;
 			 break;
-         case WHARF_LabelLocation_ID :
-             set_flags (config->set_flags, WHARF_LABEL_LOCATION);
-             config->label_location = item.data.integer;
-             break ;
-         case WHARF_AlignContents_ID :
-             set_flags (config->set_flags, WHARF_ALIGN_CONTENTS);
-             config->align_contents = ParseAlignOptions( pCurr->sub );
-             break ;
-		 case WHARF_Bevel_ID :
-             set_flags (config->set_flags, WHARF_BEVEL);
-			 clear_flags( config->flags, WHARF_NO_BORDER );
-             config->bevel = ParseBevelOptions( pCurr->sub );
-             break ;
-         case WHARF_CompositionMethod_ID :
-             set_flags (config->set_flags, WHARF_COMPOSITION_METHOD);
-             config->composition_method = item.data.integer;
-             break ;
+		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,LabelLocation ); 
+		ASCF_HANDLE_ALIGN_KEYWORD_CASE(WHARF,config,pCurr,AlignContents ); 
+		ASCF_HANDLE_BEVEL_KEYWORD_CASE(WHARF,config,pCurr,Bevel); 
+		ASCF_HANDLE_SUBSYNTAX_KEYWORD_CASE(WHARF,config,pCurr,ShowHints,WharfHints);
+		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,CompositionMethod ); 
+		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,FolderOffset ); 
+		ASCF_HANDLE_INTEGER_KEYWORD_CASE(WHARF,config,item,OrthogonalFolderOffset ); 
          case MYSTYLE_START_ID:
 			 styles_tail = ProcessMyStyleOptions (pCurr->sub, styles_tail);
 			 item.ok_to_free = 1;
@@ -750,6 +795,10 @@ SHOW_CHECKPOINT;
 				 item.ok_to_free = 1;
 		}
 	}
+	if( get_flags( config->set_flags, WHARF_Bevel ) )
+		clear_flags( config->flags, WHARF_NO_BORDER );
+		
+
 	ReadConfigItem (&item, NULL);
 SHOW_CHECKPOINT;
     DestroyConfig (ConfigReader);
@@ -759,6 +808,7 @@ SHOW_CHECKPOINT;
     return config;
 }
 
+
 #if 0
 
 FreeStorageElem **WharfFolder2FreeStorage (SyntaxDef * syntax, FreeStorageElem ** tail, WharfButton * folder, int root);
@@ -805,12 +855,12 @@ WharfButton2FreeStorage (SyntaxDef * syntax, FreeStorageElem ** tail, WharfButto
 
 				memset (&tmp, 0x00, sizeof (FunctionData));
 				tmp.func = F_Folder;
-				Func2FreeStorage (&FuncSyntax, &(new_elem->sub), &tmp);
+				Func2FreeStorage (pFuncSyntax, &(new_elem->sub), &tmp);
 				WharfFolder2FreeStorage (syntax, &(new_elem->sub->sub), button->folder, False);
 
 			} else if (button->function)
 			{
-				Func2FreeStorage (&FuncSyntax, &(new_elem->sub), button->function);
+				Func2FreeStorage (pFuncSyntax, &(new_elem->sub), button->function);
 			}
 
 		}
@@ -851,12 +901,13 @@ WriteWharfOptions (const char *filename, char *myname, WharfConfig * config, uns
 {
 	ConfigDef    *WharfConfigWriter = NULL;
 	FreeStorageElem *Storage = NULL, **tail = &Storage;
+	TermDef *folder_term = func2fterm( F_Folder, True);
 	int i ;
 
 	if (config == NULL)
 		return 1;
 
-    FuncTerms[F_Folder].sub_syntax = &WharfSyntax ;
+    folder_term->sub_syntax = &WharfSyntax ;
 
     if ((WharfConfigWriter = InitConfigWriter (myname, &WharfSyntax, CDT_Filename, (void *)filename)) == NULL)
 		return 2;
@@ -886,9 +937,6 @@ WriteWharfOptions (const char *filename, char *myname, WharfConfig * config, uns
 	/* texture_type  */
 	if (get_flags (config->set_flags, WHARF_TEXTURE_TYPE))
         tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->texture_type, WHARF_TextureType_ID);
-	/* max_colors */
-	if (get_flags (config->set_flags, WHARF_MAX_COLORS))
-        tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->max_colors, WHARF_MaxColors_ID);
 	/* animate_steps */
 	if (get_flags (config->set_flags, WHARF_ANIMATE_STEPS))
         tail = Integer2FreeStorage (&WharfSyntax, tail, NULL, config->animate_steps, WHARF_AnimateSteps_ID);
