@@ -47,6 +47,7 @@
 #include "../../libAfterStep/balloon.h"
 #include "../../libAfterStep/event.h"
 #include "../../libAfterStep/session.h"
+#include "../../libAfterStep/desktop_category.h"
 
 #include "../../libAfterConf/afterconf.h"
 
@@ -78,13 +79,11 @@ typedef struct {
 ASIdentState IdentState = {};
 
 IdentConfig *Config = NULL ;
-struct ASDatabase    *Database = NULL;
 /**********************************************************************/
 Window get_target_window();
 void GetBaseOptions (const char *filename);
 void GetOptions (const char *filename);
 void CheckConfigSanity();
-void ParseDatabase ();
 
 void HandleEvents();
 void DispatchEvent (ASEvent * event);
@@ -102,13 +101,14 @@ main( int argc, char **argv )
     /* Save our program name - for error messages */
 	set_DeadPipe_handler(DeadPipe);
     InitMyApp (CLASS_IDENT, argc, argv, NULL, NULL, 0 );
+	LinkAfterStepConfig();
 
     set_signal_handler( SIGSEGV );
 
     ConnectX( ASDefaultScr, 0 );
     ConnectAfterStep (WINDOW_CONFIG_MASK |
                       WINDOW_NAME_MASK |
-                      M_END_WINDOWLIST);
+                      M_END_WINDOWLIST, 0);
     Config = CreateIdentConfig ();
 
     /* Request a list of all windows, while we load our config */
@@ -118,7 +118,8 @@ main( int argc, char **argv )
 	LoadColorScheme();
 	LoadConfig ("ident", GetOptions);
     CheckConfigSanity();
-	ParseDatabase ();
+	ReloadASDatabase();
+	ReloadCategories(True);
 
 	if (MyArgs.src_window == 0)
 		MyArgs.src_window = get_target_window();
@@ -194,14 +195,16 @@ CheckConfigSanity()
     sprintf( buf, "*%sTile", get_application_name() );
     LOCAL_DEBUG_OUT("Attempting to use style \"%s\"", buf);
     Scr.Look.MSWindow[BACK_UNFOCUSED] = mystyle_find_or_default( buf );
-    LOCAL_DEBUG_OUT("Will use style \"%s\"", Scr.Look.MSWindow[BACK_UNFOCUSED]->name);
+    LOCAL_DEBUG_OUT("Will use style \"%s\", overlay = %p.", 
+						Scr.Look.MSWindow[BACK_UNFOCUSED]->name,
+						Scr.Look.MSWindow[BACK_UNFOCUSED]->overlay );
 }
 
 
 void
 GetBaseOptions (const char *filename)
 {
-	ReloadASEnvironment( NULL, NULL, NULL, False );
+	ReloadASEnvironment( NULL, NULL, NULL, False, False );
 }
 
 void
@@ -215,30 +218,6 @@ GetOptions (const char *filename)
     SHOW_TIME("Config parsing",option_time);
 }
 
-void
-ParseDatabase ()
-{
-    struct name_list *list = NULL ;
-	char *file = make_session_file(Session, DATABASE_FILE, False );
-	
-	/* memory management for parsing buffer */
-	if (file == NULL)
-        return ;
-
-    list = ParseDatabaseOptions (file, "afterstep");
-    if( list )
-    {
-        Database = build_asdb( list );
-        if( is_output_level_under_threshold( OUTPUT_LEVEL_DATABASE ) )
-            print_asdb( NULL, NULL, Database );
-        while (list != NULL)
-            delete_name_list (&(list));
-    }else
-        show_progress( "no database records loaded." );
-    /* XResources : */
-    load_user_database();
-}
-
 /****************************************************************************/
 /* PROCESSING OF AFTERSTEP MESSAGES :                                       */
 /****************************************************************************/
@@ -314,7 +293,8 @@ DispatchEvent (ASEvent * event)
 	        break;
 	    case PropertyNotify:
 			LOCAL_DEBUG_OUT( "property %s(%lX), _XROOTPMAP_ID = %lX, event->w = %lX, root = %lX", XGetAtomName(dpy, event->x.xproperty.atom), event->x.xproperty.atom, _XROOTPMAP_ID, event->w, Scr.Root );
-            if( event->x.xproperty.atom == _XROOTPMAP_ID && event->w == Scr.Root )
+			handle_wmprop_event (Scr.wmprops, &(event->x));
+			if( event->x.xproperty.atom == _AS_BACKGROUND )
             {
                 /*int i ;*/
                 LOCAL_DEBUG_OUT( "root background updated!%s","");
@@ -333,7 +313,6 @@ DispatchEvent (ASEvent * event)
 			{
                 /*int i ;*/
 				LOCAL_DEBUG_OUT( "AS Styles updated!%s","");
-				handle_wmprop_event (Scr.wmprops, &(event->x));
 				mystyle_list_destroy_all(&(Scr.Look.styles_list));
 				LoadColorScheme();
 				CheckConfigSanity();
@@ -352,6 +331,7 @@ make_ident_window( int width, int height)
 	int x, y ;
     XSetWindowAttributes attr;
     LOCAL_DEBUG_OUT("configured geometry is %dx%d%+d%+d", width, height, Config->geometry.x, Config->geometry.y );
+	memset( &extwm_hints, 0x00, sizeof(extwm_hints));
 	switch( Config->gravity )
 	{
 		case NorthEastGravity :
@@ -374,27 +354,29 @@ make_ident_window( int width, int height)
 	}
     attr.event_mask = StructureNotifyMask ;
     w = create_visual_window( Scr.asv, Scr.Root, x, y, width, height, 0, InputOutput, CWEventMask, &attr);
-    set_client_names( w, MyName, MyName, CLASS_IDENT, MyName );
+    set_client_names( w, MyName, MyName, AS_MODULE_CLASS, CLASS_IDENT );
 
     Scr.RootClipArea.x = x;
     Scr.RootClipArea.y = y;
     Scr.RootClipArea.width = width;
     Scr.RootClipArea.height = height;
 
-    shints.flags = USSize|PResizeInc|PWinGravity;
+    shints.flags = USSize|PMaxSize|PWinGravity;
     if( get_flags( Config->set_flags, IDENT_SET_GEOMETRY ) )
         shints.flags |= USPosition ;
     else
         shints.flags |= PPosition ;
 
-    shints.width_inc = 1;
-    shints.height_inc = 1;
+    shints.max_width = width;
+    shints.max_height = height;
 	shints.win_gravity = Config->gravity ;
 
 	extwm_hints.pid = getpid();
-    extwm_hints.flags = EXTWM_PID|EXTWM_TypeASModule ;
+    extwm_hints.flags = EXTWM_PID|EXTWM_TypeSet ;
+    extwm_hints.type_flags = EXTWM_TypeASModule ;
 
 	set_client_hints( w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
+	set_client_cmd (w);
 
 	/* showing window to let user see that we are doing something */
 	XMapRaised (dpy, w);
@@ -453,7 +435,8 @@ struct
 	char *name_set ;	  
 	char *name_unset ;	  
 } DatabaseFlagsList[] = 
-{	{STYLE_STICKY, 				"Sticky", 				"Slippery"},
+{	{STYLE_FULLSCREEN,			"Fullscreen",			"NoFullscreen"},
+	{STYLE_STICKY, 				"Sticky", 				"Slippery"},
 	{STYLE_TITLE, 				"Title", 				"NoTitle"},
 	{STYLE_CIRCULATE, 			"CirculateHit", 		"CirculateSkip"},
 	{STYLE_WINLIST, 			"WindowListHit", 		"WindowListSkip"},
@@ -472,6 +455,7 @@ struct
 	{STYLE_XRESOURCES_HINTS, 	"HonorXResources", 		"NoXResources"},
 	{STYLE_FOCUS_ON_MAP, 		"FocusOnMap", 			"NoFocusOnMap"},
 	{STYLE_LONG_LIVING,			"LongLiving",			"ShortLiving"},
+	{STYLE_IGNORE_CONFIG,  		"IgnoreConfig",			"HonorConfig"},
 	{0,NULL, NULL}};
 
 const char *Gravity2text[] =
@@ -514,15 +498,17 @@ fill_window_data()
 	name = get_window_name(wd, ASN_ResName, &encoding );
 	add_property( "Resource name:", name, encoding, False );
 
-	sprintf( buf, "%ld ( 0x%lX )", wd->client, wd->client );
+	sprintf( buf, "0x%lX ( %ld )", wd->client, wd->client );
 	add_property("Client Window ID:", buf, AS_Text_ASCII, False);
-	sprintf( buf, "%ld ( 0x%lX )", wd->frame, wd->frame );
+	sprintf( buf, "0x%lX ( %ld )", wd->frame, wd->frame );
 	add_property("Frame Window ID:", buf, AS_Text_ASCII, False);
 	sprintf( buf, "%ldx%ld%+ld%+ld", wd->frame_rect.width, wd->frame_rect.height, wd->frame_rect.x, wd->frame_rect.y );
 	add_property("Frame Geometry:", buf, AS_Text_ASCII, False);
 
 #define SHOW_FLAG(flags,flg)   \
 	do{ if( get_flags( flags, AS_##flg)){ if( buf[0] != '\0' ) strcat( buf, ", "); strcat( buf, #flg );}}while(0)
+#define SHOW_WM_FLAG(flags,flg)   \
+	do{ if( get_flags( flags, flg)){ if( buf[0] != '\0' ) strcat( buf, ", "); strcat( buf, #flg );}}while(0)
 #define SHOW_MWM_FLAG(flags,type,flg)   \
 	do{ if( get_flags( flags, MWM_##type##_##flg)){ if( buf[0] != '\0' ) strcat( buf, ", "); strcat( buf, #flg );}}while(0)
 #define SHOW_EXTWM_FLAG(flags,type,flg)   \
@@ -531,19 +517,27 @@ fill_window_data()
 	do{ if( get_flags( flags, WIN_##type##_##flg)){ if( buf[0] != '\0' ) strcat( buf, ", "); strcat( buf, #flg );}}while(0)
 
 	buf[0] = '\0' ;
+	if( (wd->state_flags & (AS_MaximizedX|AS_MaximizedY)) == (AS_MaximizedX|AS_MaximizedY) )
+	{
+		strcpy( buf, "Maximized"); 
+	}else
+	{
+		SHOW_FLAG(wd->state_flags,MaximizedX);
+		SHOW_FLAG(wd->state_flags,MaximizedY);    
+	}
 	SHOW_FLAG(wd->state_flags,Iconic);
-	SHOW_FLAG(wd->state_flags,MaximizedX);
-	SHOW_FLAG(wd->state_flags,MaximizedY);    
+	SHOW_FLAG(wd->state_flags,Fullscreen);    
 	SHOW_FLAG(wd->state_flags,Sticky);    
 	SHOW_FLAG(wd->state_flags,Shaded);        
 	SHOW_FLAG(wd->state_flags,Withdrawn);
 	SHOW_FLAG(wd->state_flags,Dead);     
 	SHOW_FLAG(wd->state_flags,Mapped);
 	SHOW_FLAG(wd->state_flags,IconMapped);
-	SHOW_FLAG(wd->state_flags,Visible);
+	SHOW_FLAG(wd->state_flags,Hidden);
 	SHOW_FLAG(wd->state_flags,Shaped);
 	SHOW_FLAG(wd->state_flags,ShapedDecor);
 	SHOW_FLAG(wd->state_flags,ShapedIcon);
+	SHOW_FLAG(wd->state_flags,Urgent);
 		   
 	add_property("Current state flags:", buf, AS_Text_ASCII, True);
 	
@@ -562,9 +556,9 @@ fill_window_data()
 	SHOW_FLAG(wd->flags,AvoidCover);          
 	SHOW_FLAG(wd->flags,IconTitle);
 	SHOW_FLAG(wd->flags,Icon);          
-	SHOW_FLAG(wd->flags,ClientIcon);
-	SHOW_FLAG(wd->flags,ClientIconPixmap);    
-	SHOW_FLAG(wd->flags,ClientIconPosition);
+	SHOW_FLAG(wd->client_icon_flags,ClientIcon);
+	SHOW_FLAG(wd->client_icon_flags,ClientIconPixmap);    
+	SHOW_FLAG(wd->client_icon_flags,ClientIconPosition);
 	SHOW_FLAG(wd->flags,FocusOnMap);          
 	SHOW_FLAG(wd->flags,ShortLived);
 	
@@ -580,6 +574,8 @@ fill_window_data()
 	SHOW_FLAG(wd->flags,BaseSize);
 	SHOW_FLAG(wd->flags,Gravity);
 	SHOW_FLAG(wd->flags,Module);
+	SHOW_FLAG(wd->flags,IgnoreConfigRequest);
+	SHOW_FLAG(wd->flags,WMDockApp);
 	
 	add_property("Specified hint values:", buf, AS_Text_ASCII, True);
 
@@ -588,6 +584,66 @@ fill_window_data()
 		ExtendedWMHints *eh = &(raw.extwm_hints);
 		GnomeHints *gh = &(raw.gnome_hints);
 		
+		if (raw.wm_hints)
+		{
+			if( raw.wm_hints->flags != 0 ) 
+			{
+				buf[0] = '\0' ;
+				SHOW_WM_FLAG(raw.wm_hints->flags,UrgencyHint );
+				SHOW_WM_FLAG(raw.wm_hints->flags,InputHint);
+				SHOW_WM_FLAG(raw.wm_hints->flags,StateHint);
+				SHOW_WM_FLAG(raw.wm_hints->flags,IconPixmapHint);
+				SHOW_WM_FLAG(raw.wm_hints->flags,IconWindowHint);
+				SHOW_WM_FLAG(raw.wm_hints->flags,IconPositionHint);
+				SHOW_WM_FLAG(raw.wm_hints->flags,IconMaskHint);
+				SHOW_WM_FLAG(raw.wm_hints->flags,WindowGroupHint);
+				add_property("WM Hints flags:", buf, AS_Text_ASCII, True);
+				if( get_flags (raw.wm_hints->flags, WindowGroupHint) )
+				{
+					sprintf( buf, "0x%lX ( %ld )", raw.wm_hints->window_group, raw.wm_hints->window_group );
+					add_property("WM Group leader:", buf, AS_Text_ASCII, True);
+				}
+			}
+		}
+		if (raw.wm_normal_hints)
+		{
+			if( raw.wm_normal_hints->flags != 0 ) 
+			{
+				buf[0] = '\0' ;
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,USPosition );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,USSize );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PPosition );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PSize );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PMinSize );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PMaxSize );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PResizeInc );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PAspect );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PBaseSize );
+				SHOW_WM_FLAG(raw.wm_normal_hints->flags,PWinGravity );
+				add_property("WM Size Hints flags:", buf, AS_Text_ASCII, True);
+			}
+			if( get_flags(raw.wm_normal_hints->flags, PWinGravity ) && 
+				raw.wm_normal_hints->win_gravity >= 0 && raw.wm_normal_hints->win_gravity <= StaticGravity ) 
+			{
+				static char *gravity_types[] = 
+					{"Forget","NorthWest","North","NorthEast","West",
+					 "Center","East","SouthWest","South","SouthEast",
+					 "Static" };
+
+					add_property("Gravity:", gravity_types[raw.wm_normal_hints->win_gravity], AS_Text_ASCII, True);
+			}
+		}
+		if( raw.transient_for)
+		{
+			sprintf( buf, "0x%lX", raw.transient_for->flags );
+			add_property("flags:", buf, AS_Text_ASCII, False);
+			sprintf( buf, "0x%lX ( %ld )", raw.transient_for->parent, raw.transient_for->parent );
+			add_property("Transient For:", buf, AS_Text_ASCII, False);
+			sprintf( buf, "%+d%+d", raw.transient_for->viewport_x, raw.transient_for->viewport_y);
+			add_property("Transient viewport:", buf, AS_Text_ASCII, False);
+			sprintf( buf, "%d", raw.transient_for->desktop );
+			add_property("Transient desktop:", buf, AS_Text_ASCII, False);
+		}
 		if (raw.motif_hints)
 		{
 			buf[0] = '\0' ;
@@ -628,29 +684,36 @@ fill_window_data()
 			}
 		}	 
 		/* window state hints : */
-		if (get_flags (eh->flags, EXTWM_StateEverything))
+		if (get_flags (eh->state_flags, EXTWM_StateEverything))
 		{
 			buf[0] = '\0' ;
-			SHOW_EXTWM_FLAG(eh->flags,State,Modal );
-			SHOW_EXTWM_FLAG(eh->flags,State,Sticky );
-			SHOW_EXTWM_FLAG(eh->flags,State,MaximizedV );
-			SHOW_EXTWM_FLAG(eh->flags,State,MaximizedH );
-			SHOW_EXTWM_FLAG(eh->flags,State,Shaded );
-			SHOW_EXTWM_FLAG(eh->flags,State,SkipTaskbar );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Modal );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Sticky );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,MaximizedV );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,MaximizedH );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Shaded );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,SkipTaskbar );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,SkipPager );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Fullscreen );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Above );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Below );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,Hidden );
+			SHOW_EXTWM_FLAG(eh->state_flags,State,DemandsAttention );
+			
 			add_property("Extended WM status flags:", buf, AS_Text_ASCII, True);
 		}
 		/* window type hints : */
-		if (get_flags (eh->flags, EXTWM_TypeEverything))
+		if (get_flags (eh->type_flags, EXTWM_TypeEverything))
 		{
 			buf[0] = '\0' ;
-			SHOW_EXTWM_FLAG(eh->flags,Type,Desktop);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Dock);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Toolbar);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Menu);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Dialog);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Normal);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Utility);
-			SHOW_EXTWM_FLAG(eh->flags,Type,Splash);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Desktop);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Dock);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Toolbar);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Menu);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Dialog);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Normal);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Utility);
+			SHOW_EXTWM_FLAG(eh->type_flags,Type,Splash);
 			add_property("Extended WM type flags:", buf, AS_Text_ASCII, True);
 		}
 		
@@ -663,6 +726,12 @@ fill_window_data()
 		if (get_flags (eh->flags, EXTWM_DoesWMPing))
 			add_property("Extended WM protocols:", "DoesWMPing", AS_Text_ASCII, False);
 		
+		if (get_flags (eh->flags, EXTWM_WINDOW_OPACITY))
+		{
+			sprintf( buf, "%ld", eh->window_opacity );
+			add_property("ExtWM Window Opacity :",  buf, AS_Text_ASCII, False);
+		}
+		
 		if (get_flags (eh->flags, EXTWM_DESKTOP))
 		{
 			if (eh->desktop == 0xFFFFFFFF)
@@ -719,10 +788,15 @@ fill_window_data()
 		add_property("Matched Styles:", "", AS_Text_ASCII, True);
 		for( i = 0 ; Database->match_list[i] >= 0 ; ++i )
 		{
-			ASDatabaseRecord *dr = &(Database->styles_table[Database->match_list[i]]) ;
+			ASDatabaseRecord *dr = 	get_asdb_record (Database, Database->match_list[i]);
 			int f ;
 			Bool first = True ;
-			sprintf( buf, "\"%s\" \t", dr->regexp->raw );
+			if( is_default_asdb_record(Database, dr) ) 
+				sprintf(buf, "\"*\" \t");
+			else if( dr->regexp ) 
+				sprintf( buf, "\"%s\" \t", dr->regexp->raw );
+			else
+				sprintf( buf, "\"(none)\" \t" );
 
 			for( f = 0 ; DatabaseFlagsList[f].name_set != NULL ; ++f ) 
 			{	
@@ -794,6 +868,20 @@ fill_window_data()
 			add_property("Style:", buf, AS_Text_ASCII, True);
 		}	 
 	}	 
+	{
+		ASDesktopEntry *de = fetch_desktop_entry( CombinedCategories, names[2]);
+		if( de == NULL ) 
+			de = fetch_desktop_entry( CombinedCategories, names[3]);				  
+		if( de ) 
+		{	
+			char *categories = make_desktop_entry_categories( de );
+			add_property("Categories:", categories?categories:"<none>", AS_Text_ASCII, False);
+			destroy_string( &categories ) ;
+			add_property(".desktop Entry Name:", de->Name, AS_Text_ASCII, False);
+			add_property(".desktop Entry Comment:", de->Comment, AS_Text_ASCII, True);
+			add_property(".desktop Entry Icon:", de->fulliconname, AS_Text_ASCII, True);
+		}
+	}
 }
 
 void 
@@ -896,6 +984,11 @@ add_property( const char *name, const char *value, unsigned long value_encoding,
 	if( IdentState.used_props < MAX_PROPERTIES )
 	{
 		int tmp ;
+		
+		if( value && strlen(value) > 0 ) 
+			printf( (strlen(name)>15)?"%s\t%s;\n":"%s\t\t%s;\n", name, value );
+		else
+			printf( "%s\n", name );
 		IdentState.props[IdentState.used_props].property_name = mystrdup(name);
 		IdentState.props[IdentState.used_props].value = mystrdup(value);
 		IdentState.props[IdentState.used_props].label_bar = create_astbar();
