@@ -50,6 +50,8 @@ typedef struct ASInternalWindow
     void (*on_hilite_changed)( struct ASInternalWindow *asiw, ASMagic *data, Bool focused );
     /* ButtonPress/Release event on one of the contexts : */
     void (*on_pressure_changed)( struct ASInternalWindow *asiw, int pressed_context );
+    /* Mouse wheel scrolling event : */
+    void (*on_scroll_event)( struct ASInternalWindow *asiw, struct ASEvent *event );
     /* Motion notify : */
     void (*on_pointer_event)( struct ASInternalWindow *asiw, struct ASEvent *event );
     /* KeyPress/Release : */
@@ -106,6 +108,7 @@ typedef struct ASWindow
 #define ASWIN_CLASS(t)      ((t)->hints->res_class)
 #define ASWIN_RES_NAME(t)   ((t)->hints->res_name)
 #define ASWIN_ICON_NAME(t)  ((t)->hints->icon_name)
+#define ASWIN_ICON_NAME_ENC(t)  (get_hint_name_encoding((t)->hints, (t)->hints->icon_name_idx))
 #define ASWIN_DESK(t)       ((t)->status->desktop)
 #define ASWIN_LAYER(t)      ((t)->status->layer)
 #define ASWIN_HFLAGS(t,f) 	get_flags((t)->hints->flags,(f))
@@ -115,6 +118,8 @@ typedef struct ASWindow
 #define ASWIN_CLEAR_FLAGS(t,f) 	clear_flags((t)->status->flags,(f))
 #define ASWIN_FUNC_MASK(t)  ((t)->hints->function_mask)
 
+#define ASWIN_FOCUSABLE(asw)  (ASWIN_HFLAGS(asw, AS_AcceptsFocus) || get_flags((asw)->hints->protocols, AS_DoesWmTakeFocus))
+
 #define IsBtnDisabled(t,b)  get_flags((t)->hints->disabled_buttons,(0x01<<(b)))
 #define IsBtnEnabled(t,b)   (!get_flags((t)->hints->disabled_buttons,(0x01<<(b))))
 #define DisableBtn(t,b)  	clear_flags((t)->hints->disabled_buttons,(0x01<<(b)))
@@ -313,6 +318,7 @@ typedef struct module_t
   int                   fd;
   int                   active;
   char                 *name;
+  char                 *cmd_line;
   CARD32                mask;
   CARD32                lock_on_send_mask;
   queue_buff_struct    *output_queue;
@@ -345,6 +351,10 @@ extern ASFlagType    AfterStepState;              /* see ASS_ flags above */
 extern struct ASDatabase *Database;
 extern ASHashTable       *ComplexFunctions;
 
+extern ASBalloonState *MenuBalloons ; 
+extern ASBalloonState *TitlebarBalloons ; 
+
+
 /* global variables for Look values : */
 extern unsigned long XORvalue;
 extern int           RubberBand;
@@ -400,6 +410,7 @@ ASWindow *bookmark2ASWindow( const char *bookmark );
 ASWindow *pattern2ASWindow( const char *pattern );
 ASWindow *complex_pattern2ASWindow( char *pattern );
 ASLayer *get_aslayer( int layer, ASWindowList *list );
+ASWindow *find_group_lead_aswindow( Window id );
 void tie_aswindow( ASWindow *t );
 void untie_aswindow( ASWindow *t );
 void add_aswindow_to_layer( ASWindow *asw, int layer );
@@ -407,6 +418,7 @@ void remove_aswindow_from_layer( ASWindow *asw, int layer );
 Bool enlist_aswindow( ASWindow *t );
 void delist_aswindow( ASWindow *t );
 void save_aswindow_list( ASWindowList *list, char *file );
+ASWindow* find_topmost_client( int desk, int root_x, int root_y );
 void restack_window_list( int desk, Bool send_msg_only );
 Bool is_window_obscured (ASWindow * above, ASWindow * below);
 void restack_window( ASWindow *t, Window sibling_window, int stack_mode );
@@ -430,7 +442,10 @@ MenuData *make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order,
 
 void hide_focus();
 Bool focus_window( ASWindow *asw, Window w );
-Bool focus_aswindow( ASWindow *asw );
+void autoraise_window( ASWindow *asw );
+#define FOCUS_ASW_CAN_AUTORAISE		False
+#define FOCUS_ASW_DONT_AUTORAISE	True
+Bool focus_aswindow( ASWindow *asw, Bool suppress_autoraise );
 Bool focus_active_window();
 void focus_next_aswindow( ASWindow *asw );     /* should be called when window is unmapped or destroyed */
 void focus_prev_aswindow( ASWindow *asw );     /* should be called when window is unmapped or destroyed */
@@ -452,13 +467,14 @@ void on_window_moveresize( ASWindow *asw, Window w );
 void on_icon_changed( ASWindow *asw );
 void on_window_title_changed( ASWindow *asw, Bool update_display );
 void on_window_hints_changed( ASWindow *asw );
+void on_window_opacity_changed( ASWindow *asw );
 void on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured );
 void on_window_hilite_changed( ASWindow *asw, Bool focused );
 void on_window_pressure_changed( ASWindow *asw, int pressed_context );
 
 void SelectDecor (ASWindow *);
 void check_aswindow_shaped( ASWindow *asw );
-ASWindow *AddWindow (Window w);
+ASWindow *AddWindow (Window w, Bool from_map_request);
 ASWindow *AddInternalWindow (Window w,
                              ASInternalWindow **pinternal,
                              struct ASHints **phints, struct ASStatusHints *status);
@@ -521,6 +537,7 @@ ASFlagType compile_titlebuttons_mask (ASHints *hints);
 void estimate_titlebar_size( ASHints *hints, unsigned int *width_ret, unsigned int *height_ret );
 void disable_titlebuttons_with_function (ASWindow * t, int function);
 Bool hints2decorations( ASWindow *asw, ASHints *old_hints );
+void invalidate_window_mystyles( ASWindow *asw );
 
 /*************************** events.c ********************************/
 const char *context2text(int ctx);
@@ -570,6 +587,7 @@ void ExecuteFunctionForClient(struct FunctionData *data, Window client);
 void ExecuteFunctionExt (struct FunctionData *data, struct ASEvent *event, int module, Bool defered);
 /* execute all the scheduled functions from the queue */
 void ExecutePendingFunctions();
+void DestroyPendingFunctionsQueue();
 /* non-window specific, non-defferrable functions are run : */
 void ExecuteBatch ( ComplexFunction *batch );
 
@@ -645,13 +663,20 @@ void broadcast_config (send_data_type event_type, ASWindow * t);
 void HandleModuleInOut(unsigned int channel, Bool has_input, Bool has_output);
 
 void KillModuleByName (char *name);
+void KillAllModulesByName (char *name);
+
 int FindModuleByName (char *name);
+char *GetModuleCmdLineByName(char *name);
 void DeadPipe (int nonsense);
 void ShutdownModules(Bool dont_free_memory);
 
 void RunCommand (FunctionData * fdata, unsigned int channel, Window w);
 
 void FlushAllQueues();
+MenuData *make_module_menu( FunctionCode func, const char *title, int sort_order );
+#define make_stop_module_menu(sort_order)  make_module_menu(  F_KILLMODULEBYNAME, "Stop Running Module", sort_order )
+#define make_restart_module_menu(sort_order)  make_module_menu(  F_RESTARTMODULEBYNAME, "Restart Running Module", sort_order )
+
 
 /******************************* outline.c ********************************/
 void MoveOutline( struct MoveResizeData * pdata );
@@ -670,12 +695,14 @@ void change_desktop_background( int desk );
 void HandleBackgroundRequest( struct ASEvent *event );
 Bool is_background_xfer_ximage( unsigned long id );
 void stop_all_background_xfer();
+void release_all_old_background( Bool forget );
 
 
 
 
 /******************************* placement.c *******************************/
 ASGrid* make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWindow *target );
+void setup_aswindow_moveresize(ASWindow *asw,  struct ASMoveResizeData *mvrdata );
 Bool place_aswindow( ASWindow *asw );
 void apply_aswindow_move(struct ASMoveResizeData *data);
 void apply_aswindow_moveresize(struct ASMoveResizeData *data);
@@ -691,11 +718,11 @@ Bool install_theme_file( const char *src );
 /******************************* winstatus.c *******************************/
 void complete_wm_state_transition( ASWindow *asw, int state );
 Bool apply_window_status_size(register ASWindow *asw, ASOrientation *od);
-Bool set_window_wm_state( ASWindow *asw, Bool iconify );
+Bool set_window_wm_state( ASWindow *asw, Bool iconify, Bool force_unmapped );
 Bool make_aswindow_visible( ASWindow *asw, Bool deiconify );
 void change_aswindow_layer( ASWindow *asw, int layer );
 void quietly_reparent_aswindow( ASWindow *asw, Window dst, Bool user_root_pos );
-void change_aswindow_desktop( ASWindow *asw, int new_desk );
+void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force );
 void toggle_aswindow_status( ASWindow *asw, ASFlagType flags );
 Bool check_window_offscreen( ASWindow *asw );
 
